/* (c) 2009 Markus Becker
 * Library fuer tog.c
 * --> Funktionen
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h> 
#include "config.h"
#include "lib.h"




ushort set_timer (ushort ms) {
	// Berechnung der Vorladezeit
	return (TIME_MAX - (ms * F_TIMER));	
}
	
void init () {
	// I/O	
	DDRD = (1 << OUT);		// PD5 am Port D als Ausgang 
	
	// Timer
	TIMSK |= (1 << TOIE1); 	// Timer Overflow Interrupt Enable (1)
	TCCR1B |= (1 << CS11) | (1 << CS10) | (1 << WGM12);	// Prescaler: 64
	TCNT1 = CNT_START; 	// Timer vorladen
	
	
	// ADC
	ADCSRA |= (1 << ADEN) | (1 << ADPS2) | (1 << ADPS0);	// AD_Enabled + Prescaler: 16 
	ADMUX |= REFS0; // AVCC als Referenz

	adc (COOL_POT);
	adc (TEMP_POT);
	LOW (OUT);
	
	t_heiz = T_HEIZ;
	
	adapt_times ();


		
}

void long_delay (ubyte ms) {
	for (; ms > 0; ms --)
		_delay_ms (1);
}

void low_out (ubyte time) {
	LOW (OUT);
	long_delay (time);
	HIGH (OUT);
}

void high_out (ubyte time) {
	HIGH (OUT);
	long_delay (time);
	LOW (OUT);
}

void startup () {
	ubyte i; // Index
	ubyte anz_startup_temps; 
	
	anz_startup_temps = T_SU_TEMPS / (2 * t_temp);
	long_delay (T_STARTUP);
	HIGH (OUT);
	long_delay (T_STARTUP);
	LOW (OUT);
	
	
	for (i = 0; i < anz_startup_temps; i ++) {
		long_delay (T_TEMP_HALB);
		HIGH (OUT);
		long_delay (T_TEMP_HALB);
		LOW (OUT);
	}
	
	// 100 ms Synchronisation
	long_delay (T_SYNC);
	
	// Interrupts annehmen + Timer starten
	TCNT1 = cnt;
	sei ();	
	
}

ubyte schalter (ubyte pin) {
	return (PIND & (1 << pin));
}

ushort adc (ubyte pin) {
	ADMUX &= 0xF0; // letzte 4 Bit loeschen
	ADMUX |= pin;
	ADCSRA |= (1<<ADSC); // ADC starten
	while (ADCSRA & (1<<ADSC) ) {
		;     // auf Abschluss der Konvertierung warten 
	}
	return ADC;  // ADC-Wert lesen
}


ushort calc_time (ushort adc, ushort min, ushort max) {
	return (min + adc * (max - min) / MAX_ADC);
}

void adapt_times (void) {
	t_temp = calc_time  (adc (TEMP_POT), MIN_TEMP_H, MAX_TEMP_H);
	t_cool = calc_time (adc (COOL_POT), MIN_COOL, MAX_COOL); 
	cnt = set_timer (t_cool); 
	t_tmp = t_temp;
}
	
